@@ -24,7 +24,7 @@ struct LatencyProbe {
 fn main() -> Result<(), Box<dyn std::error::Error>> {
     env_logger::init();
     let ditto = Ditto::builder()
-        .with_minimum_log_level(CLogLevel::Warning) // keep logs clean to see latency results on stdout
+        .with_minimum_log_level(LogLevel::Warning) // keep logs clean to see latency results on stdout
         .with_temp_dir()
         .with_identity(|ditto_root| {
             let app_id = AppId::from_env("BOUNCE_LATENCY_APP_ID")?; // Must match the ID of the Sender App
