@@ -23,7 +23,7 @@ fn main() -> Result<(), Box<dyn ErrorTrait>> {
     env_logger::init();
     let ditto = Ditto::builder()
         .with_temp_dir() // no need to store data locally
-        .with_minimum_log_level(CLogLevel::Debug)
+        .with_minimum_log_level(LogLevel::Debug)
         .with_identity(|ditto_root| {
             let auth_url = env::var("DITTO_AUTH_URL").ok();
             let app_id = AppId::from_env("DITTO_APP_ID")?;
