@@ -1,9 +1,14 @@
 use serde::Serialize;
+use std::ops::Not;
 
 use crate::{
     error::{DittoError, ErrorKind},
     ffi_sdk,
-    store::{collection::document_id::DocumentId, update::UpdateResult},
+    store::{
+        collection::document_id::DocumentId,
+        live_query::{SingleDocumentEventHandler, SingleDocumentLiveQueryEvent},
+        update::UpdateResult,
+    },
     subscription::Subscription,
     utils::prelude::*,
 };
@@ -119,11 +124,13 @@ impl PendingIdSpecificOperation {
     }
 
     /// Enables you to listen for changes that occur in relation to a document.
-    /// The eventHandler closure will be called when local or remote changes
-    /// are made to the document referenced by the findByID call that
-    /// precedes the call to observe. The returned DittoLiveQuery object
-    /// must be kept in scope for as long as you want the provided eventHandler
-    /// to be called when an update occurs.
+    /// The `handler` closure will be called when local or remote changes are
+    /// made to the document referenced by the `find_by_id` call that precedes
+    /// the call to `observe`.
+    ///
+    /// The returned `LiveQuery` object must be kept in scope for as long as you
+    /// want the provided `handler` to be called when an update occurs.
+    #[deprecated(note = "Use `observe_v2` instead")]
     pub fn observe<Handler>(&self, handler: Handler) -> Result<LiveQuery, DittoError>
     where
         Handler: EventHandler,
@@ -132,13 +139,15 @@ impl PendingIdSpecificOperation {
     }
 
     /// Enables you to listen for changes that occur in relation to a document.
-    /// This won’t subscribe to receive changes made remotely by others and
-    /// so it will only fire updates when a local change is made. If you
-    /// want to receive remotely performed updates as well then use
-    /// observe or also call subscribe separately after another findByID call
-    /// that references the same document ID. The returned DittoLiveQuery
-    /// object must be kept in scope for as long as you want the provided
-    /// eventHandler to be called when an update occurs.
+    /// This won’t subscribe to receive changes made remotely by others and so
+    /// it will only fire updates when a local change is made. If you want to
+    /// receive remotely performed updates as well then use `observe` or also
+    /// call `subscribe` separately after another `find_by_id` call that
+    /// references the same document ID.
+    ///
+    /// The returned `LiveQuery` object must be kept in scope for as long as you
+    /// want the provided `handler` to be called when an update occurs.
+    #[deprecated(note = "Use `observe_local_v2` instead")]
     pub fn observe_local<Handler>(&self, handler: Handler) -> Result<LiveQuery, DittoError>
     where
         Handler: EventHandler,
@@ -146,6 +155,36 @@ impl PendingIdSpecificOperation {
         self.observe_internal(None, handler)
     }
 
+    /// Enables you to listen for changes that occur in relation to a document.
+    /// The `handler` closure will be called when local or remote changes are
+    /// made to the document referenced by the `find_by_id` call that precedes
+    /// the call to `observe`.
+    ///
+    /// The returned `LiveQuery` object must be kept in scope for as long as you
+    /// want the provided `handler` to be called when an update occurs.
+    pub fn observe_v2<Handler>(&self, handler: Handler) -> Result<LiveQuery, DittoError>
+    where
+        Handler: SingleDocumentEventHandler,
+    {
+        self.observe_internal_v2(Some(Box::new(self.subscribe())), handler)
+    }
+
+    /// Enables you to listen for changes that occur in relation to a document.
+    /// This won’t subscribe to receive changes made remotely by others and so
+    /// it will only fire updates when a local change is made. If you want to
+    /// receive remotely performed updates as well then use `observe` or also
+    /// call `subscribe` separately after another `find_by_id` call that
+    /// references the same document ID.
+    ///
+    /// The returned `LiveQuery` object must be kept in scope for as long as you
+    /// want the provided `handler` to be called when an update occurs.
+    pub fn observe_local_v2<Handler>(&self, handler: Handler) -> Result<LiveQuery, DittoError>
+    where
+        Handler: SingleDocumentEventHandler,
+    {
+        self.observe_internal_v2(None, handler)
+    }
+
     fn observe_internal<Handler>(
         &self,
         sub: Option<Box<Subscription>>,
@@ -167,6 +206,55 @@ impl PendingIdSpecificOperation {
         )
     }
 
+    fn observe_internal_v2<Handler>(
+        &self,
+        sub: Option<Box<Subscription>>,
+        handler: Handler,
+    ) -> Result<LiveQuery, DittoError>
+    where
+        Handler: SingleDocumentEventHandler,
+    {
+        let mut handler = handler;
+        let mapped_handler = move |mut docs: Vec<BoxedDocument>, event: LiveQueryEvent| match event
+        {
+            LiveQueryEvent::Initial => {
+                if docs.len() > 1 {
+                    ::log::error!("Single document live query returned more than 1 document");
+                    debug_assert!(docs.len() <= 1);
+                }
+                let event = SingleDocumentLiveQueryEvent {
+                    is_initial: true,
+                    old_document: None,
+                };
+                handler(docs.pop(), event);
+            }
+            LiveQueryEvent::Update {
+                mut old_documents,
+                insertions,
+                deletions,
+                updates,
+                moves,
+            } => {
+                if moves.is_empty().not() {
+                    ::log::error!("Single document live query received an update event saying that there was a move, which should not happen");
+                    debug_assert!(moves.is_empty());
+                }
+                if insertions.len() > 1 || deletions.len() > 1 || updates.len() > 1 {
+                    ::log::error!("Single document live query received an update event with too many insertions, deletions, or updates");
+                    debug_assert!(insertions.len() <= 1);
+                    debug_assert!(deletions.len() <= 1);
+                    debug_assert!(updates.len() <= 1);
+                }
+                let event = SingleDocumentLiveQueryEvent {
+                    is_initial: false,
+                    old_document: old_documents.pop(),
+                };
+                handler(docs.pop(), event);
+            }
+        };
+        self.observe_internal(sub, mapped_handler)
+    }
+
     /// Update the document with the matching ID.
     /// * `updater` - a Fn which will be called on the selected document if
     ///   found
@@ -285,20 +373,9 @@ impl PendingIdSpecificOperation {
 
 #[cfg(test)]
 mod tests {
-
     use serde_json::json;
 
-    use crate::{error::DittoError, prelude::*, utils::prelude::ErrorKind};
-
-    fn setup_ditto() -> Result<Ditto, DittoError> {
-        let ditto = Ditto::builder()
-            .with_temp_dir()
-            .with_identity(|ditto_root| identity::OfflinePlayground::random(ditto_root))?
-            .with_minimum_log_level(CLogLevel::Info)
-            .build()?;
-        ditto.set_license_from_env("DITTO_LICENSE")?;
-        Ok(ditto)
-    }
+    use crate::{prelude::*, test_helpers::setup_ditto, utils::prelude::ErrorKind};
 
     #[test]
     fn test_doc_not_found() {
@@ -321,8 +398,7 @@ mod tests {
         let store = ditto.store();
         let collection = store.collection("test").unwrap();
         let doc_id = DocumentId::new(&"test_key").unwrap();
-        let doc_id_1 = doc_id.clone();
-        let doc = collection.find_by_id(doc_id).exec();
+        let doc = collection.find_by_id(doc_id.clone()).exec();
 
         assert!(doc.is_err());
 
@@ -330,12 +406,12 @@ mod tests {
         let error_kind = e.kind();
         assert_eq!(error_kind, ErrorKind::NonExtant);
 
-        let content = json!({"hello": "again"});
-        let inserted = collection.insert(content, Some(&doc_id_1), false);
+        let content = json!({"hello": "again", "_id": doc_id});
+        let inserted = collection.upsert(content);
 
         assert!(inserted.is_ok());
 
-        let doc = collection.find_by_id(doc_id_1).exec();
+        let doc = collection.find_by_id(doc_id).exec();
 
         assert!(doc.is_ok());
     }
