@@ -1,17 +1,29 @@
 use_prelude!();
 use ::ffi_sdk::{self, c_cb_params, COrderByParam, LiveQueryAvailability};
 pub use event::LiveQueryEvent;
+pub use single_document_event::SingleDocumentLiveQueryEvent;
 
 use crate::{
     error::{DittoError, ErrorKind},
     subscription::Subscription,
 };
 mod event;
+mod single_document_event;
 
 pub use move_module::LiveQueryMove;
 #[path = "move.rs"]
 mod move_module;
 
+trait_alias! {
+    /// A closure which is called on each event for a single document live query
+    /// (`find_by_id(...).observe(...)`)
+    pub
+    trait SingleDocumentEventHandler =
+        FnMut(Option<BoxedDocument>, SingleDocumentLiveQueryEvent)
+        + Send // can be dropped in another thread
+        + 'static // cannot dangle
+}
+
 trait_alias! {
     /// A closure which is called on each event
     pub
