@@ -66,8 +66,13 @@ impl PresenceManagerV2Context {
         PeersObserver::new(arc_context)
     }
 
+    pub(crate) fn parse_presence(presence_json_str: &str) -> Result<V2Presence, String> {
+        serde_json::from_str(presence_json_str)
+            .map_err(|e| format!("Error deserializing V2Presence, error = {:?}", e))
+    }
+
     pub(crate) fn on_presence(&self, presence_json_str: &str) {
-        let event: V2Presence = match serde_json::from_str(presence_json_str) {
+        let event = match Self::parse_presence(presence_json_str) {
             Ok(event) => event,
             Err(e) => {
                 ::log::error!("Error deserializing V2Presence, error = {:?}", e);
@@ -151,10 +156,14 @@ pub struct V2Peer {
     /// The operating system of a peer (if known).
     pub os: Option<V2Os>,
 
-    /// An optional mesh role which an app developer might provide
+    /// An optional query overlap group which an app developer might provide
     /// to influence connection priorities. Values can range between
     /// 0-63 (inclusive). Defaults to `0` if not set.
     #[serde(default)]
+    pub query_overlap_group: u8,
+
+    #[deprecated(note = "Use `query_overlap_group` instead")]
+    #[serde(default)]
     pub mesh_role: u8,
 
     /// Flag which indicates if this peer is connected to HyDRA. This is
@@ -274,3 +283,120 @@ pub enum V2ConnectionType {
     P2PWiFi,
     WebSocket,
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    const V2_PEER_JSON: &str = r#"
+            {
+                "localPeer": 1,
+                "peers": [
+                    {
+                        "id": 1,
+                        "siteId": "1",
+                        "deviceName": "local-peer",
+                        "os": "macOS",
+                        "isHydraConnected": false,
+                        "isCompatible": true,
+                        "dittoSdkVersion": "1.0.0",
+                        "meshRole": 0,
+                        "queryOverlapGroup": 0
+                    },
+                    {
+                        "id": 2,
+                        "siteId": "2",
+                        "deviceName": "device-2",
+                        "os": "iOS",
+                        "isHydraConnected": false,
+                        "isCompatible": true,
+                        "dittoSdkVersion": null,
+                        "meshRole": 0,
+                        "queryOverlapGroup": 0
+                    },
+                    {
+                        "id": 3,
+                        "siteId": "3",
+                        "deviceName": "device-3",
+                        "os": "Android",
+                        "isHydraConnected": false,
+                        "isCompatible": true,
+                        "dittoSdkVersion": "1.0.3",
+                        "meshRole": 32,
+                        "queryOverlapGroup": 32
+                    },
+                    {
+                        "id": 4,
+                        "siteId": "4",
+                        "deviceName": "device-4",
+                        "os": "Linux",
+                        "isHydraConnected": false,
+                        "isCompatible": true,
+                        "dittoSdkVersion": null
+                    }
+                ],
+                "connections": [
+                    {
+                        "id": "1<->2:Bluetooth",
+                        "from": 1,
+                        "to": 2,
+                        "connectionType": "Bluetooth",
+                        "approximateDistanceInMeters": 2.2963063716888428
+                    },
+                    {
+                        "id": "1<->3:AccessPoint",
+                        "from": 1,
+                        "to": 3,
+                        "connectionType": "AccessPoint",
+                        "approximateDistanceInMeters": null
+                    },
+                    {
+                        "id": "1<->4:WebSocket",
+                        "from": 1,
+                        "to": 4,
+                        "connectionType": "WebSocket",
+                        "approximateDistanceInMeters": null
+                    }
+                ]
+            }
+        "#;
+
+    #[test]
+    fn test_json_parsing() {
+        let presence = PresenceManagerV2Context::parse_presence(V2_PEER_JSON).unwrap();
+        let peers = presence.peers;
+
+        assert_eq!(peers.len(), 4);
+
+        // Local Peer
+        assert_eq!(peers[0].id, 1);
+        assert_eq!(peers[0].device_name, "local-peer");
+        assert_eq!(peers[0].os, Some(V2Os::MacOS));
+        assert_eq!(peers[0].query_overlap_group, 0);
+
+        // Remote Peers
+
+        assert_eq!(peers[1].id, 2);
+        assert_eq!(peers[1].device_name, "device-2");
+        assert_eq!(peers[1].os, Some(V2Os::Ios));
+        assert_eq!(peers[1].query_overlap_group, 0);
+
+        assert_eq!(peers[2].id, 3);
+        assert_eq!(peers[2].device_name, "device-3");
+        assert_eq!(peers[2].os, Some(V2Os::Android));
+        assert_eq!(peers[2].query_overlap_group, 32);
+
+        assert_eq!(peers[3].id, 4);
+        assert_eq!(peers[3].device_name, "device-4");
+        assert_eq!(peers[3].os, Some(V2Os::Linux));
+        assert_eq!(peers[3].query_overlap_group, 0);
+
+        #[allow(deprecated)]
+        {
+            assert_eq!(peers[0].mesh_role, 0); // 0 in JSON
+            assert_eq!(peers[1].mesh_role, 0); // 0 in JSON
+            assert_eq!(peers[2].mesh_role, 32); // 32 in JSON
+            assert_eq!(peers[3].mesh_role, 0); // Missing in JSON
+        }
+    }
+}
