@@ -30,7 +30,7 @@ fn everything() -> Result<(), Box<dyn std::error::Error>> {
         ..Default::default()
     };
 
-    let redhonda_id = Arc::new(Mutex::new(col1.insert(doc1, None, false)?));
+    let redhonda_id = Arc::new(Mutex::new(col1.upsert(doc1)?));
 
     let doc2 = TestType {
         make: String::from("Honda"),
@@ -38,7 +38,7 @@ fn everything() -> Result<(), Box<dyn std::error::Error>> {
         ..Default::default()
     };
 
-    let _crimsonhonda_id = Arc::new(col1.insert(doc2, None, false)?);
+    let _crimsonhonda_id = Arc::new(col1.upsert(doc2)?);
 
     let doc3 = TestType {
         make: String::from("Bentley"),
@@ -51,8 +51,8 @@ fn everything() -> Result<(), Box<dyn std::error::Error>> {
         ..Default::default()
     };
 
-    let redbentley_id = Arc::new(Mutex::new(col2.insert(doc3, None, false)?));
-    let redjaguar_id = Arc::new(Mutex::new(col2.insert(doc4, None, false)?));
+    let redbentley_id = Arc::new(Mutex::new(col2.upsert(doc3)?));
+    let redjaguar_id = Arc::new(Mutex::new(col2.upsert(doc4)?));
 
     let redtoyota_id = Arc::new(Mutex::new(DocumentId::default()));
     let redford_id = Arc::new(Mutex::new(DocumentId::default()));
@@ -84,6 +84,7 @@ fn everything() -> Result<(), Box<dyn std::error::Error>> {
                 let expected_doc = TestType {
                     make: String::from("Honda"),
                     color: TestColor::Red,
+                    id: Some(redhonda_id.lock().unwrap().clone()),
                     ..Default::default()
                 };
                 assert_eq!(first_doc, expected_doc);
@@ -93,7 +94,7 @@ fn everything() -> Result<(), Box<dyn std::error::Error>> {
                     make: String::from("Toyota"),
                     ..Default::default()
                 };
-                *redtoyota_id.lock().unwrap() = collection.insert(redtoyota, None, false).unwrap();
+                *redtoyota_id.lock().unwrap() = collection.upsert(redtoyota).unwrap();
             }
             LiveQueryEvent::Update {
                 moves,
@@ -117,8 +118,7 @@ fn everything() -> Result<(), Box<dyn std::error::Error>> {
                                 color: TestColor::Red,
                                 ..Default::default()
                             };
-                            *redford_id.lock().unwrap() =
-                                collection.insert(redford, None, false).unwrap();
+                            *redford_id.lock().unwrap() = collection.upsert(redford).unwrap();
                         }
                         3 => {
                             assert_eq!(docs.len(), 3);
@@ -164,11 +164,9 @@ fn everything() -> Result<(), Box<dyn std::error::Error>> {
                                 ..Default::default()
                             };
                             *orangemazda_id.lock().unwrap() =
-                                collection.insert(orangemazda, None, false).unwrap();
-                            *yellowam_id.lock().unwrap() =
-                                collection.insert(yellowam, None, false).unwrap();
-                            *redchevy_id.lock().unwrap() =
-                                collection.insert(redchevy, None, false).unwrap();
+                                collection.upsert(orangemazda).unwrap();
+                            *yellowam_id.lock().unwrap() = collection.upsert(yellowam).unwrap();
+                            *redchevy_id.lock().unwrap() = collection.upsert(redchevy).unwrap();
                         }
                         5 => {
                             assert_eq!(docs.len(), 4);
@@ -213,9 +211,9 @@ fn everything() -> Result<(), Box<dyn std::error::Error>> {
                                         ..Default::default()
                                     };
                                     *redhyundai_id.lock().unwrap() =
-                                        col_tx.insert(redhyundai, None, false).unwrap().clone();
+                                        col_tx.upsert(redhyundai).unwrap().clone();
                                     *pinkjeep_id.lock().unwrap() =
-                                        col_tx.insert(pinkjeep, None, false).unwrap().clone();
+                                        col_tx.upsert(pinkjeep).unwrap().clone();
                                     // remove the red toyota
                                     col_tx
                                         .find_by_id(redtoyota_id.lock().unwrap().clone())
@@ -264,7 +262,7 @@ fn everything() -> Result<(), Box<dyn std::error::Error>> {
                                         ..Default::default()
                                     };
                                     *blackbmw_id.lock().unwrap() =
-                                        col2_tx.insert(blackbmw, None, false).unwrap().clone();
+                                        col2_tx.upsert(blackbmw).unwrap().clone();
                                     col2_tx
                                         .find_by_id(redbentley_id.lock().unwrap().clone())
                                         .evict()
@@ -330,8 +328,7 @@ fn everything() -> Result<(), Box<dyn std::error::Error>> {
                                 color: TestColor::Red,
                                 ..Default::default()
                             };
-                            *redferrari_id.lock().unwrap() =
-                                collection.insert(redferrari, None, false).unwrap();
+                            *redferrari_id.lock().unwrap() = collection.upsert(redferrari).unwrap();
                         }
                         8 => {
                             assert_eq!(docs.len(), 4);
