@@ -208,7 +208,7 @@ mod snippets {
         let collection = ditto.store().collection("cars").unwrap();
         //@ditto/snippet-start query-basic
         collection
-            .find("favoriteBooks[0].title == \'The Great Gatsby\'")
+            .find("favoriteBook.title == \'The Great Gatsby\'")
             .exec()?;
         //@ditto/snippet-end
 
